------------------------------------------------------------------------
-- @class Branch
-- Class representing a player's branch
-- Branch of the player, can grow or or decay or do whatever branches do
------------------------------------------------------------------------

class "Branch"{
}

function Branch:__init__(p_parent, p_hook)
    print("new branch")
    self.parent     =   p_parent
    self.species    =   p_parent.species
    self.image      =   self.species.branch_image
    self.empty_image=   self.species.branch_image_empty
    self.anchor     =   {}
    self.hook       =   {}
    self.branches   =   {}
    self.level      =   0
    self.angle      =   0
    self.time       =   0
    self.opacity    =   255
    self.transparent=   self.parent.transparent or false
    self.hook.x, self.hook.y        =   unpack(self.species.branch_hook)
    self.anchor.x, self.anchor.y    =   unpack(self.species.branch_anchor)
    self.x_pos, self.y_pos          =   0,0
    if p_hook then
        self.x_pos, self.y_pos      =   p_hook.x, p_hook.y --if several hooks, get the good one
    else
        self.x_pos, self.y_pos      =   p_parent.hook.x, p_parent.hook.y
    end
end

function Branch:setTransparent(p_bool)
    self.transparent    =   p_bool

end

function Branch:update(dt)
    self.time   =   self.time + dt
    if self.transparent then
        self.opacity    =   self.opacity - 500 * dt
    else
        self.opacity    =   self.opacity + 500 * dt
    end

    --~ if self.transparent then
        --~ self.opactiy    =   self.opacity - dt * 500
        --~ print("more")
    --~ else
        --~ self.opactiy    =   self.opacity + dt * 500
        --~ print("less")
    --~ end
--~
    if self.opacity > 255 then self.opacity = 255 end
    if self.opacity < 0 then self.opacity = 0 end
end

function Branch:levelUp()
    self.level  =   self.level + 1
    self.parent:levelUp()
end

function Branch:confirmBranch(p_ghost)
    local b = Branch(self)
    b:fromGhost(p_ghost)
    table.insert(self.branches, b)
    table.insert(self:getTree().branches, b)
    table.insert(GAME_OBJECTS,1, b)
    self:levelUp()
    print("GameObject has now: "..#GAME_OBJECTS)
end

function Branch:draw()
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.draw(self:getDrawable(self.empty_image))

    if self.opacity ~= 0 then --If fully transparent do not draw
        love.graphics.setColor(255, 255, 255, self.opacity)
        love.graphics.draw(self:getDrawable())
    end

    if DEBUG then
        love.graphics.setColor(255,0,0)
        love.graphics.circle("fill",self.anchor.x, self.anchor.y, 10)
        love.graphics.setColor(0,255,0)
        love.graphics.circle("fill",self.x_pos, self.y_pos, 10)
        love.graphics.setColor(0,0,255)
        love.graphics.circle("fill", self.hook.x, self.hook.y, 10)
        love.graphics.setColor(255,255,255)
    end
end

function Branch:fromGhost(p_ghost)
    self.anchor =   p_ghost.anchor
    self.x_pos  =   p_ghost.x_pos
    self.y_pos  =   p_ghost.y_pos
    self.angle  =   p_ghost.angle
    local v_hook  =   vector(self.hook.x - self.anchor.x, self.hook.y - self.anchor.y)
    v_hook:rotate_inplace(self.angle)
    self.hook.x, self.hook.y    =   v_hook.x+self.x_pos, v_hook.y+self.y_pos

end

function Branch:getDrawable(p_image)
    local scale =   1 + (self.level/50)
    if not p_image then

        return self.image, self.x_pos, self.y_pos, self.angle, scale, scale, self.anchor.x, self.anchor.y
    else
        return p_image, self.x_pos, self.y_pos, self.angle, scale, scale, self.anchor.x, self.anchor.y
    end

end

function Branch:getTree()
    return self.parent:getTree()
end

function Branch:appendBranch()
    local t = self:getTree()
    local b = GhostBranch(self)

    t.appending_element =   b
    t.appending         =   true

    --Correct this

    print("Seed is inserting the dummy in GUI")
    table.insert(GAME_OBJECTS, b)
    print("Gui has now: "..#GUI)
    b=nil


end

function Branch:getActions()
    if #self.branches >= self.species.max_branches then
        return {"Leaf", "Decay"}
    else
        return {"Branch","Leaf","Decay"}
    end
end

function Branch:testPoint(p_x, p_y)
    print("Testing point on branch with angle: "..self.angle)
    -- TODO: a real collision testing algo

    -- accepts a point (x, y), aabb center (rx, ry), aabb half-width/half-height (hw, hh) and an angle in radians (a)
    --~ point_vs_raabb ( x, y, rx, ry, hw, hh, a )
    local test = point_vs_raabb (p_x, p_y, self.x_pos, self.y_pos, self.image:getWidth()/2, self.image:getHeight()/2, self.angle)
    print("Test result for branch: ")
    print(test)
    return test

    --Temp: check if clicked near the anchor (<=100px)
    --~ local distance  =   math.sqrt((self.x_pos - p_x)^2+(self.y_pos - p_y)^2)
    --~ return (distance < 100)

    -- Assumes rectancle is ABCD check if ABCD area is inferior to sum of triangles area

end
