require "lib.slither"
require "lib.resource_loader"
require "class.Seed"
require "class.Core"
require "class.Config"
require "class.Rectangle"
require "class.Button"
require "class.Level"
require "class.Tree"
require "class.Branch"
require "class.GhostBranch"
require "class.RadialMenu"


require "data.i18n.translator"
require "data.i18n.fr"
Gamestate = require "lib.hump.gamestate"
ringbuffer= require "lib.hump.ringbuffer"
camera    = require "lib.hump.camera"
vector    = require "lib.hump.vector"

dofile("lib/table.save.lua")
require "menu_gamestates"
require "play_gamestates"

APP_NAME        =   "Seed"
PROJECT_VERSION =   " alpha 0.0.1"


function love.load()
    -- Init gamestates and display
    Gamestate.registerEvents()
    print(APP_NAME..PROJECT_VERSION)
    love.graphics.setCaption(APP_NAME..PROJECT_VERSION)
    GAMECONFIG      =   Config()
    setLocale(GAMECONFIG.options["locale"])

    -- Record the gametime
    gametime        =   0


    -- Loading fonts
    title_font      =   love.graphics.newFont("res/fonts/Varling.ttf", 89)
    big_font        =   love.graphics.newFont("res/fonts/Varling.ttf", 42)
    standard_font   =   love.graphics.newFont("res/fonts/Varling.ttf", 24)
    love.graphics.setFont(standard_font)


    -- Loading species
    load_species()

    --Switching to default mode: newgame
    Gamestate.switch(splashscreen)



end

function love.keyreleased(key)
    if key=="d" then
        DEBUG   =   not DEBUG
    end

end

function love.draw()
    love.graphics.setFont(standard_font)
    love.graphics.setColor(255, 255, 255)

end

function love.update(dt)
    gametime    =   gametime + dt

end

function love.quit()
  print("Played "..gametime.." seconds, leaving game...")

end

-- Util function to recursively print userdata
function tprint (tbl, indent)
  if not indent then indent = 0 end
  for k, v in pairs(tbl) do
    formatting = string.rep("  ", indent) .. k .. ": "
    if type(v) == "table" then
      print(formatting)
      tprint(v, indent+1)
    else
      print(tostring(formatting) .. tostring(v))
    end
  end
end


function debug_draw_stats(p_cam)
    love.graphics.setFont(standard_font)
    local  msg  =   "FPS:"..love.timer.getFPS()
    msg = msg.."\nMouse position:\ncam:("..love.mouse.getX()..","..love.mouse.getY()..")"
    if p_cam then
        local x, y = p_cam:worldCoords(love.mouse.getPosition())
        msg =   msg.."\nworld:("..math.floor(x)..","..math.floor(y)..")"
    end
    love.graphics.print(msg, 10, 50)
end

function debug_draw_shapes(p_group)
    for i,obj in pairs(p_group) do
        if obj.hook then
            love.graphics.setColor(100, 255, 100)
            love.graphics.circle("fill", obj.hook.x, obj.hook.y, 10)
        end
        if obj.anchor then
            love.graphics.setColor(255, 100, 100)
            love.graphics.circle("fill", obj.anchor.x, obj.anchor.y, 10)
        end
    end
    love.graphics.setColor(255, 255, 255)
end

function math.clamp(input, min_val, max_val)
    if input < min_val then
        input = min_val
    elseif input > max_val then
        input = max_val
    end
    return input
end
