------------------------------------------------------------------------
-- @file menu_gamestates.lua
-- File storing the differents menu gamestates and their callbacks
------------------------------------------------------------------------



-- States: SEEDING, MENU, SURFACE, GROUND, PAUSE

splashscreen    =   Gamestate.new()
newgame         =   Gamestate.new()
mainmenu        =   Gamestate.new()
optionmenu      =   Gamestate.new()


TRANSITION              =   {}
TRANSITION.ONGOING      =   false
TRANSITION.TIME         =   1.2
TRANSITION.ELAPSED      =   0
TRANSITION.SOUND_IN     =   love.audio.newSource("res/sounds/transition_in.ogg", "static")
TRANSITION.SOUND_OUT    =   love.audio.newSource("res/sounds/transition_out.ogg", "static")


clicksound              =   love.audio.newSource("res/sounds/button_onClick.ogg", "static")
mainmenu_music          =   love.audio.newSource("res/music/mainmenu.ogg")
mainmenu_music:setLooping(true)
--~ mainmenu_music:setLooping(true)

-- Positions for differents menu items
splashscreen.position   =   vector(love.graphics.getWidth()/2, love.graphics.getHeight()/2)
splashscreen.rotation   =   0
splashscreen.zoom       =   1
mainmenu.position       =   vector(love.graphics.getWidth()/2, love.graphics.getHeight()/15*10) --2 third, center
mainmenu.rotation       =   0
mainmenu.zoom           =   0.3
optionmenu.position     =   vector(1050, 1050)
optionmenu.rotation     =   28.1
optionmenu.zoom         =   1
newgame.position        =   vector(300, -550)
newgame.rotation        =   -155
newgame.zoom            =   1

------------------------------------------------------------------------
-- SPLASHSCREEN displays a header and waits for user input
------------------------------------------------------------------------
function splashscreen:init()
    menucam             =   camera(love.graphics.getWidth()/2, love.graphics.getHeight()/2)
    splash_image        =   love.graphics.newImage("res/images/gui/splash.png")-- Splash image
    splash_image_pos    =   vector(love.graphics.getWidth()/2-splash_image:getWidth()/2, love.graphics.getHeight()/2-splash_image:getHeight()/2)
    love.audio.play(mainmenu_music)

end

function splashscreen:enter()
    TRANSITION.ONGOING      =   false
    TRANSITION.ELAPSED      =   0

end

function splashscreen:draw()
    menucam:attach()

    --Drawing splash image
    love.graphics.draw(splash_image, splash_image_pos:unpack())
    menucam:detach()

    if not TRANSITION.ONGOING then
        love.graphics.printf(_("Press any key to continue..."), 0, love.graphics.getHeight()/5*4, love.graphics.getWidth(), "center")
    end

end

function splashscreen:update(dt)
    if TRANSITION.ONGOING then
        TRANSITION.ELAPSED  =   TRANSITION.ELAPSED + dt
        if not direction then
            local len_vector=   mainmenu.position - splashscreen.position
            speed           =   len_vector:len() / (TRANSITION.TIME)
            direction       =   len_vector:normalized()
            zoomspeed       =   (mainmenu.zoom - menucam.zoom) / (TRANSITION.TIME)
            rotspeed        =   math.rad(mainmenu.rotation) / (TRANSITION.TIME)
        else
            if TRANSITION.ELAPSED<TRANSITION.TIME then
                menucam:move((direction*speed*dt):unpack())
                menucam:rotate(rotspeed*dt)
                menucam.zoom = menucam.zoom + zoomspeed*dt
            else
                --~ print("TODO switch to main menu")
                Gamestate.switch(mainmenu)
            end
        end
    end
    menucam:move(math.sin(gametime)/8,math.cos(gametime)/8)

end

function splashscreen:keyreleased(key)
    if key=="escape" then
        love.event.quit()
    else
        TRANSITION.ONGOING  =   true
        love.audio.stop(TRANSITION.SOUND_OUT)
        love.audio.play(TRANSITION.SOUND_OUT)
    end
end

function splashscreen:mousereleased(x, y, button)
    if button=="l" or button=="r" then
        TRANSITION.ONGOING  =   true
        love.audio.stop(TRANSITION.SOUND_OUT)
        love.audio.play(TRANSITION.SOUND_OUT)
    end
end


------------------------------------------------------------------------
-- MAINMENU - allows the player to start or load a game and such
------------------------------------------------------------------------
function mainmenu:init()


end

function mainmenu:enter(previous)
    menucam.rot         =   math.rad(mainmenu.rotation)
    menucam.zoom        =   mainmenu.zoom
    TRANSITION.ONGOING  =   false
    TRANSITION.ELAPSED  =   0

    item_list   =   {"Resume game", "New game", "Options", "Quit"}
    line_offset =   (love.graphics.getHeight()/2) / #item_list
    line_offset =   line_offset -   standard_font:getHeight()
    menu        =   {}

    ypos  =   love.graphics.getHeight()/2
    -- Building the menu items
    for k,v in pairs(item_list) do
        local menuitem  =   {}
        local width =   big_font:getWidth(string.gsub(_(v)," ","_"))
        local height=   big_font:getHeight(string.gsub(_(v)," ","_"))
        local xpos  =   love.graphics.getWidth()/2 - width/2

        menuitem.rect   =   Rectangle(xpos, ypos, width, height)
        menuitem.label  =   item_list[k]
        menuitem.hover  =   false
        --~ love.graphics.printf(v, xpos, ypos, width, "center")
        ypos = ypos+line_offset
        function menuitem:onClick()
            love.audio.stop(clicksound)
            love.audio.play(clicksound)
            if menuitem.label=="Quit" then
                love.event.quit()
            elseif menuitem.label=="Options" then
                --~ Gamestate.switch(optionmenu)
                TRANSITION.ONGOING  =   true
                love.audio.stop(TRANSITION.SOUND_IN)
                love.audio.play(TRANSITION.SOUND_IN)
                local len_vector=   optionmenu.position - mainmenu.position
                speed           =   len_vector:len() / (TRANSITION.TIME)
                direction       =   len_vector:normalized()
                zoomspeed       =   (optionmenu.zoom - menucam.zoom) / (TRANSITION.TIME)
                rotspeed        =   math.rad(optionmenu.rotation - menucam.rot) / (TRANSITION.TIME)
                nextmenu        =   optionmenu

            elseif menuitem.label=="New game" then
                --~ Gamestate.switch(newgame)
                TRANSITION.ONGOING  =   true
                love.audio.stop(TRANSITION.SOUND_IN)
                love.audio.play(TRANSITION.SOUND_IN)
                local len_vector=   newgame.position - mainmenu.position
                speed           =   len_vector:len() / (TRANSITION.TIME)
                direction       =   len_vector:normalized()
                zoomspeed       =   (newgame.zoom - menucam.zoom) / (TRANSITION.TIME)
                rotspeed        =   math.rad(newgame.rotation - menucam.rot) / (TRANSITION.TIME)
                nextmenu        =   newgame
                print("going to new")
            elseif menuitem.label=="Resume game" then
                print("Todo ! ")
            end
        end
        table.insert(menu, menuitem)
    end


end

function mainmenu:update(dt)
    menucam:move(math.sin(gametime)/8,math.cos(gametime*0.5)/8)
    if TRANSITION.ONGOING then
        TRANSITION.ELAPSED = TRANSITION.ELAPSED + dt
        if TRANSITION.ELAPSED < TRANSITION.TIME then
            --~ menucam.zoom = menucam.zoom + mainmenu.to_options_transition_zoom_speed
            menucam:move((direction*speed*dt):unpack())
            menucam:rotate(rotspeed*dt)
            menucam.zoom = menucam.zoom + zoomspeed*dt
        else
            tprint(nextmenu)
            Gamestate.switch(nextmenu)
        end
    else
        for k,v in pairs(menu) do
            x, y = love.mouse.getPosition()
            if v.rect:contains(x, y) then
                v.hover =   true
            else
                v.hover =   false
            end
            --~ love.graphics.printf(v.label, v.x, v.y, v.width, "center")
        end
    end
end

function mainmenu:draw()
    menucam:attach()
    --Drawing splash image
    local x_pos = love.graphics.getWidth()/2 - splash_image:getWidth()/2
    local y_pos = love.graphics.getHeight()/2 - splash_image:getHeight()/2
    love.graphics.draw(splash_image, x_pos, y_pos)
    menucam:detach()
    if not TRANSITION.ONGOING then
        love.graphics.setFont(big_font)
        for k,v in pairs(menu) do
            if v.hover then
                love.graphics.setColor( 100, 200, 100)
            else
                love.graphics.setColor( 255, 255, 255)
            end
            love.graphics.printf(_(v.label), v.rect.x, v.rect.y, v.rect.width, "center")
        end
    end
end

function mainmenu:keyreleased(key, unicode)
    if key=="return" then
        Gamestate.switch(newgame)
    end
end

function mainmenu:mousereleased(m_x, m_y, button)
    for k,v in pairs(menu) do
        if v.rect:contains(m_x, m_y) then
            v:onClick()
        end
    end
end


------------------------------------------------------------------------
-- OPTIONS MENU - allows to change sound and display parameters
------------------------------------------------------------------------
function optionmenu:init()


end

function optionmenu:enter()
    menucam.rot         =   math.rad(optionmenu.rotation)
    menucam.zoom        =   optionmenu.zoom
    TRANSITION.ONGOING  =   false
    TRANSITION.ELAPSED  =   0

    --~ item_list   =   ringbuffer("Resume game", "New game", "Options", "Quit")
    item_list   =   {"Display", "Sound", "Language", "Back"}
    line_offset =   (love.graphics.getHeight()/2) / #item_list
    line_offset =   line_offset -   standard_font:getHeight()
    menu        =   {}

    ypos  =   love.graphics.getHeight()/2
    -- Building the menu items
    for k,v in pairs(item_list) do
        local menuitem  =   {}
        local width =   big_font:getWidth(string.gsub(_(v)," ","_"))
        local height=   big_font:getHeight(string.gsub(_(v)," ","_"))
        local xpos  =   love.graphics.getWidth()/2 - width/2

        menuitem.rect   =   Rectangle(xpos, ypos, width, height)
        menuitem.label  =   item_list[k]
        menuitem.hover  =   false
        ypos = ypos+line_offset
        function menuitem:onClick()
            love.audio.stop(clicksound)
            love.audio.play(clicksound)
            if menuitem.label=="Language" then
                if GAMECONFIG.options["locale"]=="en" then
                    GAMECONFIG.options["locale"]="fr"
                else
                    GAMECONFIG.options["locale"]="en"
                end
                setLocale(GAMECONFIG.options["locale"])
                GAMECONFIG:saveToFile()
            elseif menuitem.label=="Back" then
                love.audio.stop(TRANSITION.SOUND_OUT)
                love.audio.play(TRANSITION.SOUND_OUT)
                TRANSITION.ONGOING  =   true
                love.audio.stop(TRANSITION.SOUND_OUT)
                love.audio.play(TRANSITION.SOUND_OUT)
                local len_vector=   mainmenu.position - optionmenu.position
                speed           =   len_vector:len() / (TRANSITION.TIME)
                direction       =   len_vector:normalized()
                zoomspeed       =   (mainmenu.zoom - menucam.zoom) / (TRANSITION.TIME)
                rotspeed        =   math.rad(optionmenu.rotation - menucam.rot) / (TRANSITION.TIME)
            end
        end
        table.insert(menu, menuitem)
    end

end

function optionmenu:update(dt)
    menucam:move(math.sin(gametime)/8,math.cos(gametime*0.5)/8)
    if TRANSITION.ONGOING then
        TRANSITION.ELAPSED = TRANSITION.ELAPSED + dt
        if TRANSITION.ELAPSED < TRANSITION.TIME then
            --~ menucam.zoom = menucam.zoom + mainmenu.to_options_transition_zoom_speed
            menucam:move((direction*speed*dt):unpack())
            menucam:rotate(-rotspeed*dt)
            menucam.zoom = menucam.zoom + zoomspeed*dt
        else
            Gamestate.switch(mainmenu)
        end
    else
        for k,v in pairs(menu) do
            x, y = love.mouse.getPosition()
            if v.rect:contains(x, y) then
                v.hover =   true
            else
                v.hover =   false
            end
        end
    end
end

function optionmenu:draw()
    menucam:attach()
    --Drawing splash image
    local x_pos = love.graphics.getWidth()/2 - splash_image:getWidth()/2
    local y_pos = love.graphics.getHeight()/2 - splash_image:getHeight()/2
    love.graphics.draw(splash_image, x_pos, y_pos)

    menucam:detach()
    if not TRANSITION.ONGOING then
        love.graphics.setFont(big_font)
        for k,v in pairs(menu) do
            if v.hover then
                love.graphics.setColor( 100, 200, 100)
            else
                love.graphics.setColor( 255, 255, 255)
            end
            love.graphics.printf(_(v.label), v.rect.x, v.rect.y, v.rect.width, "center")
        end
    end

end

function optionmenu:keyreleased(key, unicode)
    if key=="escape" then
        TRANSITION.ONGOING  =   true
        love.audio.stop(TRANSITION.SOUND_OUT)
        love.audio.play(TRANSITION.SOUND_OUT)
        local len_vector=   mainmenu.position - optionmenu.position
        speed           =   len_vector:len() / (TRANSITION.TIME)
        direction       =   len_vector:normalized()
        zoomspeed       =   (mainmenu.zoom - menucam.zoom) / (TRANSITION.TIME)
        rotspeed        =   math.rad(optionmenu.rotation - menucam.rot) / (TRANSITION.TIME)
    end
end

function optionmenu:mousereleased(m_x, m_y, button)
    for k,v in pairs(menu) do
        if v.rect:contains(m_x, m_y) then
            v:onClick()
        end
    end
end
------------------------------------------------------------------------
-- NEW GAME
------------------------------------------------------------------------
function newgame:init() -- run only once
    print("Entering NEW GAME state")
    print("Loading seed images")
    species =   load_species()
    rb  =   ringbuffer(0)
    for k,v in pairs(species) do
        rb:insert(v)
    end
    rb:remove()


    -- Creating buttons
    h = love.graphics.getHeight()/2
    w = love.graphics.getWidth()/2
    padding = 200

    l = love.graphics.newImage("res/images/sprites/left-arrow.png") -- Default image
    r = love.graphics.newImage("res/images/sprites/right-arrow.png") -- Default image
    right_arrow =   Button()
    right_arrow:setImage(r)
    right_arrow:setCenter(w + padding, h)
    right_arrow.onClick = function() rb:next() end

    left_arrow  =   Button()
    left_arrow:setImage(l)
    left_arrow:setCenter(w - padding, h)
    left_arrow.onClick = function() rb:prev() end

    play_button         =   {}
    play_button.rect    =   Rectangle(0,0,150,45)
    play_button.rect:setCenter(w, h + padding*1.5)
    play_button.label   =   "Play"

    font = love.graphics.newFont("res/fonts/Varling.ttf", 35)
    love.graphics.setFont(font)
    print("Choosing menu is ready")
end

function newgame:update(dt)
    menucam:move(math.sin(gametime)/8,math.cos(gametime*0.5)/8)
    if TRANSITION.ONGOING then
        TRANSITION.ELAPSED = TRANSITION.ELAPSED + dt
        if TRANSITION.ELAPSED < TRANSITION.TIME then
            --~ menucam.zoom = menucam.zoom + mainmenu.to_options_transition_zoom_speed
            menucam:move((direction*speed*dt):unpack())
            menucam:rotate(-rotspeed*dt)
            menucam.zoom = menucam.zoom + zoomspeed*dt
        else
            Gamestate.switch(optionmenu)
        end
    end
end

function newgame:enter(previous) -- run every time the state is entered
    menucam.rot         =   math.rad(newgame.rotation)
    menucam.zoom        =   newgame.zoom
    TRANSITION.ONGOING  =   false
    TRANSITION.ELAPSED  =   0
    --~ print("newgame: enter")
end

function newgame:draw()
    menucam:attach()
    --Drawing splash image
    local x_pos = love.graphics.getWidth()/2  - splash_image:getWidth() /2
    local y_pos = love.graphics.getHeight()/2 - splash_image:getHeight()/2
    love.graphics.draw(splash_image, x_pos, y_pos)

    menucam:detach()

    mid_x = love.graphics.getWidth()  / 2
    mid_y = love.graphics.getHeight() / 2


    rect_w= 256
    rect_h= 256

    -- Draw the frame
    love.graphics.setColor(10, 155, 10, 120)
    love.graphics.rectangle( "fill", mid_x - rect_w/2, mid_y - rect_h/2, rect_w, rect_h )

    love.graphics.setColor(55, 55, 55, 100)
    rect_w=rect_w/1.025
    rect_h=rect_h/1.025
    love.graphics.rectangle( "fill", mid_x - rect_w/2, mid_y - rect_h/2, rect_w, rect_h )


    -- Draw the image of the seed
    rect_w= rb:get().seed_image:getWidth()
    rect_h= rb:get().seed_image:getHeight()
    love.graphics.setColor(255, 255, 255)
    love.graphics.draw(rb:get().seed_image, mid_x, mid_y, 0, 1, 1, rect_w/2, rect_h/2)


    -- Draw the buttons
    love.graphics.draw(left_arrow:getDrawable())
    love.graphics.draw(right_arrow:getDrawable())
    love.graphics.rectangle(play_button.rect:getDrawable())
    love.graphics.printf(_(play_button.label), play_button.rect.x, play_button.rect.y, play_button.rect.width, "center")


    -- Draw the text
    love.graphics.setFont(title_font)
    love.graphics.printf(_(rb:get().name), 0, 80, love.graphics.getWidth(), "center")
    love.graphics.setFont(standard_font)
    desc_string =   _("Description")..":\t".._(rb:get().description).."\n"
    .._("Jumps")..":\t"..rb:get().max_jumps.."\n"
    .."Max branches:\t"..rb:get().max_branches .."\n"
    .."Max roots:\t"..rb:get().max_roots

    love.graphics.printf(_(desc_string), 200, 550, love.graphics.getWidth(), "left")



end

function newgame:keyreleased(key)
    if key == "right" then
        rb:next()
    end

    if key == "left" then
        rb:prev()
    end

    if key == "escape" then
        love.event.quit()
    end

    if key == "return" then
        print("entering game...")
        print("Chosen species: "..rb:get().name)
        seeding.species =   rb:get()
        Gamestate.switch(seeding)
    end
end

function newgame:mousereleased(m_x, m_y, button)
    print("click at: "..m_x.." "..m_y)
    if button == "l" then
        if left_arrow:contains(m_x, m_y) then
            rb:prev()
        elseif right_arrow:contains(m_x, m_y) then
            rb:next()
        elseif play_button.rect:contains(m_x, m_y) then
            seeding.species =   rb:get()
            Gamestate.switch(seeding)
        end

    end

end

function track_point(p_cam, point)

    c_x, c_y    =   p_cam:pos()
    x_speed, y_speed    =   0, 0
    offset      =   75
    speed       =   2.75

    y_distance  =   math.abs(c_y - point.y)
    x_distance  =   math.abs(c_x - point.x)


    -- Speed of the cam
    if y_distance > offset then
        y_speed =   speed * math.log(y_distance-offset)
    end

    if x_distance > offset then
        x_speed =   speed * math.log(x_distance-offset)
    end

    -- Direction of the cam
    if c_x > point.x then
        x_speed =   -x_speed
    end
    if c_y > point.y then
        y_speed =   -y_speed
    end
    p_cam:move(x_speed, y_speed)

end
